/*
   This file shows how a Java program can redirect
   its standard input & output streams to files.
   The methods that do this are
      System.setIn()  and  System.setOut()

   After this program redirects stdin & stdout, it just
   reads lines from stdin and echoes them to stdout.

   After the I/O redirection, this program's standard
   streams are connected as in the following picture.

             RedirectStandardStreams
             +----------------+
             |                |
 data.txt--->>stdin    stdout>>-----result.txt
             |                |
             |         stderr>>-----console window
             |                |
             +----------------+

   Be sure you understand what happens when you execute the
   following command-line. Notice that in this command-line,
   the shell program does I/O redirections of the standard
   streams for this program (and then this program does its
   own redirection of its own streams!).

   C:\> java RedirectStandardStreams < Echo.java > something.txt


   Important: Notice how this program is similar to the
   following command line.

   C:\> java Echo < RedirectStandardStreams.java > result.txt

   In the case of this command line, the shell program sets up the
   I/O redirections before running the program Echo.class. In the
   case of this program, the program itself does its own I/O
   redirections.
*/
import java.util.Scanner;
import java.io.*;

public class RedirectStandardStreams
{
   public static void main(String[] args) throws FileNotFoundException
   {
      // Set System.in to be a file (instead of whatever stdin originally was).
      System.setIn( new FileInputStream( "RedirectStandardStreams.java" ) );

      // Set System.out to be a file (instead of whatever stdout originally was).
      System.setOut( new PrintStream( new FileOutputStream( "result.txt" ) ) );

      // Create a Scanner object to make it easier to use System.in
      Scanner scanner = new Scanner( System.in );

      // Echo every line of input from stdin to stdout.
      while ( scanner.hasNextLine() )
      {
         String oneLine = scanner.nextLine();
         // If you want to, you could manipulate (i.e., filter)
         // the line of input before sending it to stdout.
         System.out.println( oneLine );
      }
   }
}
