/*
   Create the following parent to child pipeline.

            parent                        child
       +-------------+               +-------------+
       |             |     pipe      |             |
   --->>0          1>>--0========0-->>0          1>>------->
       |             |               |             |
       |           2>>-----+         |           2>>--+-->
       |             |     |         |             |  |
       +-------------+     |         +-------------+  |
                           |                          |
                           +--------------------------+
*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/wait.h>

#define  STDIN_PIPE  0 /* pipe index for the fd to read from */
#define STDOUT_PIPE  1 /* pipe index for the fd to write to  */

int main()
{
   int pipefd[2]; /* the pipe */
   int fork_rv;

   /* create a pipe */
   if ( pipe(pipefd) == -1 )
   {
      perror("pipe");
   }

   fork_rv = fork();    /* create the child process */
   if ( fork_rv == -1 ) /* check for error */
   {
      perror("fork");
   }
   else if ( fork_rv > 0 )  /* parent process */
   {
      /* parent calls close(3), close(1), dup(4), close(4) */
      close( pipefd[STDIN_PIPE] );       /* pipefd[0] == 3 */
      close(STDOUT_FILENO);              /* close(1)       */
      dup( pipefd[STDOUT_PIPE] );        /* pipefd[1] == 4 */
      close( pipefd[STDOUT_PIPE] );      /* pipefd[1] == 4 */

      /* do a simple filtering operation on stdin and write results to stdout. */
      char c;
      while ( (c = getchar()) != EOF )
         if ( 'a' <= c && c <= 'z' ) /* from lower case */
         {
            printf("%c", c - 32);    /* to upper case */
         }
         else
         {
            printf("%c", c);
            fflush(stdout);  // try commenting this out
         }

      /* Close the input to the pipe (STDOUT_FILENO) so that
         the child process stops reading and terminates. */
      close(STDOUT_FILENO);

      /* wait for the child process */
      wait(NULL);
   }
   else  /* child process */
   {
      /* child calls close(4), close(0), dup(3), close(3) */
      close( pipefd[STDOUT_PIPE] );     /* pipefd[0] == 4 */
      close(STDIN_FILENO);              /* close(0)       */
      dup( pipefd[STDIN_PIPE] );        /* pipefd[1] == 3 */
      close( pipefd[STDIN_PIPE] );      /* pipefd[1] == 3 */

      /* child becomes the second stage program */
      execlp("./double", NULL);
      perror("execlp child1");
      exit(1);
   }
   return 0;
}
